#include "CommandShm.hpp"

#include <boost/interprocess/shared_memory_object.hpp>
#include <boost/make_shared.hpp>

#include <iostream>

const char *CommandShm::s_segment_suffix        = "_CommandMemory";
const char *CommandShm::s_command_struct_suffix = "_CommandStruct";

using namespace boost::interprocess;

CommandShm::CommandShm() {

}

CommandShm::~CommandShm() noexcept {

}

//! this should only be called by the server
//! \throw bad_alloc on fail
CommandInterfaceSPtr CommandShm::create(const char *n_service_name) {

	assert(n_service_name);

	CommandInterfaceSPtr ret(boost::make_shared<CommandInterface>());
	ret->m_segment_name.assign(n_service_name).append(CommandShm::s_segment_suffix);
	ret->m_command_struct_name.assign(n_service_name).append(CommandShm::s_command_struct_suffix);

	shared_memory_object::remove(ret->m_segment_name.c_str());

	// Create a new segment with given name and size
	boost::interprocess::managed_shared_memory *segment = new managed_shared_memory(create_only, ret->m_segment_name.c_str(), CommandShm::total_size);
	ret->m_command_segment = segment;

	// Create an object in shared memory
	ret->m_command_shm = segment->construct<CommandShm>(ret->m_command_struct_name.c_str())();

	// create the messages maps
	ret->m_command_shm->m_commands = segment->construct<MessageMap>("CommandMap")      // object name should be unnecessary here
			(std::less<boost::uint64_t>()		   // first  ctor parameter
			, segment->get_segment_manager());     // second ctor parameter

	ret->m_command_shm->m_responses = segment->construct<MessageMap>("ResponseMap")
			(std::less<boost::uint64_t>()
			, segment->get_segment_manager());

	return ret;
};

//! \throw bad_alloc on fail
CommandInterfaceSPtr CommandShm::open(const char *n_service_name) noexcept {

	try {
		assert(n_service_name);

		CommandInterfaceSPtr ret(boost::make_shared<CommandInterface>());
		ret->m_segment_name.assign(n_service_name).append(CommandShm::s_segment_suffix);
		ret->m_command_struct_name.assign(n_service_name).append(CommandShm::s_command_struct_suffix);

		// Open a new segment with given name and size
		boost::interprocess::managed_shared_memory  *segment = new managed_shared_memory(open_only, ret->m_segment_name.c_str());
		ret->m_command_segment = segment;
		
		// open the object in shared memory
		std::pair<CommandShm *, std::size_t>  segpair = segment->find<struct CommandShm>(ret->m_command_struct_name.c_str());
		ret->m_command_shm = segpair.first;

		// theoretically I should assert the message maps 
		// but I recall that you cannot safely test an offset_ptr to null

		return ret;

	} catch (const std::exception &sex) {
		std::cerr << "Error opening shared memory segment: " << sex.what() << std::endl;
		return nullptr;
	}
};
