//  Copyright 2016 (c) Stephan Menzel, vr-on GmbH

#include "ShmClient.hpp"
#include "CommandShm.hpp"

#include <boost/cstdint.hpp>
#include <boost/thread/mutex.hpp>
#include <boost/thread/thread.hpp>
#include <boost/date_time/posix_time/ptime.hpp>

#include <exception>
#include <cassert>

using namespace boost::interprocess;

ShmClient::ShmClient()
		: m_command(CommandShm::open("test")) {

	if (!m_command) {
		throw std::bad_alloc();
	}
}

ShmClient::~ShmClient() noexcept {

}

std::string ShmClient::send_command(const std::string &n_command) noexcept {

	assert(m_command && m_command->m_command_shm);

	std::string response;
	boost::uint64_t id = 42;   // originally random
	CommandShm *cbuffer = m_command->m_command_shm;

	// first I put my command into the shm command map along with a 
	// random id that will tell me I got a response
	{
		scoped_lock<interprocess_mutex> lock(cbuffer->m_command_mutex);
		msgmap_insert(cbuffer->m_commands.get(), m_command, id, n_command);
		cbuffer->m_command_condition.notify_all();
	}

	// now I wait on the response map for a response with my ID
	bool response_received = false;
	while (!response_received) {
		scoped_lock<interprocess_mutex> lock(cbuffer->m_response_mutex);
		if (cbuffer->m_responses->count(id) == 0) {
			// no response yet, continue waiting 
			cbuffer->m_response_condition.timed_wait(lock, boost::get_system_time() + boost::posix_time::milliseconds(100));
		} else {
			// got a response. copy out of the map and delete it
			MessageMap::iterator i = cbuffer->m_responses->find(id);

			assert(i != cbuffer->m_responses->end());
			response.assign(i->second->cbegin(), i->second->cend());
	
			m_command->m_command_segment->get_segment_manager()->destroy_ptr(i->second.get());
			cbuffer->m_responses->erase(i);
			response_received = true;
		}
	}

	return response;
}
